#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
Name: setup.py
Porpose: script to setup Videomass.
Compatibility: Python3
Platform: all
Writer: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Oct.22.2021
Code checker: flake8, pylint

This file is part of Videomass.

    Videomass is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Videomass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
import platform
from setuptools import setup, find_packages
from videomass.vdms_sys.msg_info import current_release
from videomass.vdms_sys.msg_info import descriptions_release


def source_build():
    """
    Source/Build distributions

    """
    # Get info data
    crel = current_release()
    drel = descriptions_release()

    if 'sdist' in sys.argv or 'bdist_wheel' in sys.argv:

        inst_req = ["wxpython>=4.0.7; platform_system=='Windows' or "
                    "platform_system=='Darwin'",
                    "PyPubSub>=4.0.3",
                    "yt_dlp>=2023.03.04",
                    "requests>=2.26.0",
                    ]
        setup_req = ["setuptools>=47.1.1",
                     "wheel>=0.34.2",
                     "twine>=3.1.1"
                     ]
        with open('README.md', 'r', encoding='utf-8') as readme:
            long_descript = readme.read()

        long_description_ct = 'text/markdown'

    else:  # e.g. to make a Debian source package, include wxpython.
        inst_req = ["wxpython>=4.0.7",
                    "PyPubSub>=4.0.3",
                    "requests>=2.26.0",
                    ]
        setup_req = []
        long_descript = drel[1]
        long_description_ct = 'text'

    excluded = ['']
    # pathnames must be relative-path
    if platform.system() == 'Windows':
        data_f = [('share/pixmaps', ['videomass/art/icons/videomass.png'])]

    elif platform.system() == 'Darwin':
        data_f = [('share/pixmaps', ['videomass/art/icons/videomass.png']),
                  ('share/man/man1', ['docs/man/man1/videomass.1.gz']),
                  ]
    else:
        data_f = [('share/applications',
                   ['videomass/art/io.github.jeanslack.videomass.desktop']),
                  ('share/metainfo',
                   ['io.github.jeanslack.videomass.appdata.xml']),
                  ('share/pixmaps', ['videomass/art/icons/videomass.png']),
                  ('share/icons/hicolor/48x48/apps',
                   ['videomass/art/icons/hicolor/48x48/apps/videomass.png',
                    'videomass/art/icons/hicolor/48x48/apps/videomass.xpm']),
                  ('share/icons/hicolor/256x256/apps',
                   ['videomass/art/icons/hicolor/256x256/apps/videomass.png']
                   ),
                  ('share/icons/hicolor/scalable/apps',
                   ['videomass/art/icons/hicolor/scalable/apps/'
                    'videomass.svg']),
                  ('share/man/man1', ['docs/man/man1/videomass.1.gz']),
                  ]
    setup(name=crel[1],
          version=crel[2],
          description=drel[0],
          long_description=long_descript,
          long_description_content_type=long_description_ct,
          author=crel[6][0],
          author_email=crel[7],
          url=crel[5],
          license=drel[2],
          platforms=["All"],
          packages=find_packages(exclude=excluded),
          data_files=data_f,
          package_data={"videomass": ["art/icons/*", "locale/*"]
                        },
          exclude_package_data={"videomass": ["art/videomass.icns",
                                              "art/videomass.ico",
                                              "locale/README",
                                              "locale/videomass.pot"
                                              ]
                                },
          include_package_data=True,
          zip_safe=False,
          python_requires=">=3.7.0, <3.12.0",
          install_requires=inst_req,
          setup_requires=setup_req,
          entry_points={'gui_scripts':
                        ['videomass = videomass.gui_app:main']},
          classifiers=[
        'Environment :: X11 Applications :: GTK',
        'Development Status :: 5 - Production/Stable',
        'Operating System :: MacOS',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: Chinese (Simplified)',
        'Natural Language :: Dutch',
        'Natural Language :: English',
        'Natural Language :: French',
        'Natural Language :: Italian',
        'Natural Language :: Portuguese (Brazilian)',
        'Natural Language :: Russian',
        'Natural Language :: Spanish',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.7',
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        'Topic :: Multimedia :: Video',
        'Topic :: Multimedia :: Video :: Conversion',
        'Topic :: Multimedia :: Sound/Audio :: Conversion',
    ],
    )


if __name__ == '__main__':
    source_build()



# -*- coding: UTF-8 -*-

# Porpose: Contains test cases for the Videomass3 object.
# Rev: Oct.04.2020 *PEP8 compatible*

import sys
import os.path
import unittest

if sys.version_info[0] != 3:
    sys.exit('\nERROR: You are using an unsupported version of Python. '
             'Python3 is required..\n')

PATH = os.path.realpath(os.path.abspath(__file__))
sys.path.insert(0, os.path.dirname(os.path.dirname(PATH)))

try:
    import wx

except ImportError as error:
    sys.exit(error)

else:
    from videomass import gui_app


class GuiTestCase(unittest.TestCase):
    """
    Test case for Videomass GUI. It tests the bootstrap of
    the wxPython module by starting Videomass app.
    """

    def setUp(self):
        """Method called to prepare the test fixture"""
        kwargs = {'make_portable': None}
        self.app = gui_app.Videomass(redirect=False, **kwargs)

    def tearDown(self):
        """
        start MainLoop and destroy
        see:
            - https://github.com/wxWidgets/Phoenix/blob/master/unittests/wtc.py
            - https://stackoverflow.com/questions/33292441/how-to-destroy-a-wxpython-frame-in-unittest
        """
        def _cleanup():
            for tlw in wx.GetTopLevelWindows():
                if tlw:
                    # tlw.Close(force=True)
                    tlw.Destroy()
            wx.WakeUpIdle()

        # timer = wx.PyTimer(_cleanup)
        # timer.Start(100)
        wx.CallLater(100, _cleanup)
        self.app.MainLoop()
        del self.app

    def test_app(self):
        """test app"""
        if self.app:
            self.assertTrue(self.app)


def main():
    unittest.main()


if __name__ == '__main__':
    main()



# -*- coding: UTF-8 -*-
"""
Name: ffmpeg_codecs.py
Porpose: Mini-frame to show the available decoders/encoders on the FFmpeg
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Feb.13.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx
from pubsub import pub


class FFmpegCodecs(wx.Dialog):
    """
    It shows a dialog box with a pretty kind of GUI to view
    the formats available on FFmpeg
    """
    def __init__(self, dict_decoders, OS, option):
        """
        with 'None' not depend from parent:
        wx.Dialog.__init__(self, None, style=wx.DEFAULT_DIALOG_STYLE)

        With parent, -1:
        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        if close videomass also close parent window:
        """
        self.option = option
        if self.option == '-encoders':
            title = _("FFmpeg encoders")
            header = _("supported encoders")
        else:
            title = _("FFmpeg decoders")
            header = _("supported decoders")
        get = wx.GetApp()  # get data from bootstrap
        vidicon = get.iconset['videomass']

        wx.Dialog.__init__(self, None,
                           style=wx.DEFAULT_DIALOG_STYLE
                           | wx.RESIZE_BORDER
                           | wx.DIALOG_NO_PARENT
                           )
        # add panel
        self.panel = wx.Panel(self, wx.ID_ANY,
                              style=wx.TAB_TRAVERSAL
                              | wx.BORDER_THEME,
                              )
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        notebook = wx.Notebook(self.panel, wx.ID_ANY,)
        sizer_base.Add(notebook, 1, wx.ALL | wx.EXPAND, 5)
        # -- nb 1
        nb_panel_1 = wx.Panel(notebook, wx.ID_ANY)
        vid = wx.ListCtrl(nb_panel_1, wx.ID_ANY,
                          style=wx.LC_REPORT
                          | wx.SUNKEN_BORDER,
                          )
        sizer_tab1 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab1.Add(vid, 1, wx.ALL | wx.EXPAND, 5)
        nb_panel_1.SetSizer(sizer_tab1)
        notebook.AddPage(nb_panel_1, (_("Video")))
        # --- nb2
        nb_panel_2 = wx.Panel(notebook, wx.ID_ANY)
        aud = wx.ListCtrl(nb_panel_2, wx.ID_ANY,
                          style=wx.LC_REPORT
                          | wx.SUNKEN_BORDER,
                          )
        sizer_tab2 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab2.Add(aud, 1, wx.ALL | wx.EXPAND, 5)
        nb_panel_2.SetSizer(sizer_tab2)
        notebook.AddPage(nb_panel_2, (_("Audio")))
        # -- nb3
        nb_panel_3 = wx.Panel(notebook, wx.ID_ANY)
        sub = wx.ListCtrl(nb_panel_3, wx.ID_ANY,
                          style=wx.LC_REPORT
                          | wx.SUNKEN_BORDER,
                          )
        sizer_tab3 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab3.Add(sub, 1, wx.ALL | wx.EXPAND, 5)
        nb_panel_3.SetSizer(sizer_tab3)
        notebook.AddPage(nb_panel_3, (_("Subtitle")))
        # ----- text
        stext = wx.StaticText(self.panel, wx.ID_ANY, "")
        sizer_base.Add(stext, 0, wx.ALL | wx.EXPAND, 5)
        # ----- btns
        button_close = wx.Button(self.panel, wx.ID_CLOSE, "")
        grid_buttons = wx.GridSizer(1, 1, 0, 0)
        grid_buttons.Add(button_close, 1, wx.ALL, 5)
        sizer_base.Add(grid_buttons, flag=wx.ALIGN_RIGHT | wx.RIGHT, border=0)

        # ----------------------Properties----------------------#
        vid.InsertColumn(0, header, width=200)
        vid.InsertColumn(1, ('F'), width=40)
        vid.InsertColumn(2, ('S'), width=40)
        vid.InsertColumn(3, ('X'), width=40)
        vid.InsertColumn(4, ('B'), width=40)
        vid.InsertColumn(5, ('D'), width=40)
        vid.InsertColumn(6, _('description'), width=450)
        # vid.SetBackgroundColour(wx.Colour(217, 255, 255))
        # aud.SetMinSize((600, 300))
        aud.InsertColumn(0, header, width=200)
        aud.InsertColumn(1, ('F'), width=40)
        aud.InsertColumn(2, ('S'), width=40)
        aud.InsertColumn(3, ('X'), width=40)
        aud.InsertColumn(4, ('B'), width=40)
        aud.InsertColumn(5, ('D'), width=40)
        aud.InsertColumn(6, _('description'), width=450)
        # aud.SetBackgroundColour(wx.Colour(217, 255, 255))
        # sub.SetMinSize((600, 300))
        sub.InsertColumn(0, header, width=200)
        sub.InsertColumn(1, ('F'), width=40)
        sub.InsertColumn(2, ('S'), width=40)
        sub.InsertColumn(3, ('X'), width=40)
        sub.InsertColumn(4, ('B'), width=40)
        sub.InsertColumn(5, ('D'), width=40)
        sub.InsertColumn(6, _('description'), width=450)
        # sub.SetBackgroundColour(wx.Colour(217, 255, 255))

        if OS == 'Darwin':
            vid.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            aud.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            sub.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            stext.SetFont(wx.Font(11, wx.SWISS, wx.ITALIC, wx.NORMAL))
        else:
            vid.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            aud.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            sub.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            stext.SetFont(wx.Font(8, wx.SWISS, wx.ITALIC, wx.NORMAL))

        leg = ("F = frame-level multithreading\n"
               "S = slice-level multithreading\n"
               "X = Codec is experimental\n"
               "B = Supports draw_horiz_band\n"
               "D = Supports direct rendering method 1")
        stext.SetLabel(leg)

        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(vidicon, wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        self.SetTitle(title)
        self.SetMinSize((750, 500))
        vid.SetMinSize((700, 300))
        self.panel.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()

        # populate vid listctrl output:
        index = 0
        vcodlist = dict_decoders['Video']
        if vcodlist:
            for a in vcodlist:
                vid.InsertItem(index, a[6:].split(' ')[1])
                if 'F' in a[1]:
                    vid.SetItem(index, 1, 'YES')
                if 'S' in a[2]:
                    vid.SetItem(index, 2, 'YES')
                if 'X' in a[3]:
                    vid.SetItem(index, 3, 'YES')
                if 'B' in [4]:
                    vid.SetItem(index, 4, 'YES')
                if 'D' in [5]:
                    vid.SetItem(index, 5, 'YES')
                d = " ".join(a.split()).split(None, 2)[2]
                if len(d):
                    vid.SetItem(index, 6, d)
                else:
                    vid.SetItem(index, 6, '')
                index += 1

        # populate aud listctrl output:
        index = 0
        acodlist = dict_decoders['Audio']
        if acodlist:
            for a in acodlist:
                aud.InsertItem(index, a[6:].split(' ')[1])
                if 'F' in a[1]:
                    aud.SetItem(index, 1, 'YES')
                if 'S' in a[2]:
                    aud.SetItem(index, 2, 'YES')
                if 'X' in a[3]:
                    aud.SetItem(index, 3, 'YES')
                if 'B' in [4]:
                    aud.SetItem(index, 4, 'YES')
                if 'D' in [5]:
                    aud.SetItem(index, 5, 'YES')
                d = " ".join(a.split()).split(None, 2)[2]
                if len(d):
                    aud.SetItem(index, 6, d)
                else:
                    aud.SetItem(index, 6, '')
                index += 1

        # populate sub listctrl output:
        index = 0
        scodlist = dict_decoders['Subtitle']
        if scodlist:
            for a in scodlist:
                sub.InsertItem(index, a[6:].split(' ')[1])
                if 'F' in a[1]:
                    sub.SetItem(index, 1, 'YES')
                if 'S' in a[2]:
                    sub.SetItem(index, 2, 'YES')
                if 'X' in a[3]:
                    sub.SetItem(index, 3, 'YES')
                if 'B' in [4]:
                    sub.SetItem(index, 4, 'YES')
                if 'D' in [5]:
                    sub.SetItem(index, 5, 'YES')
                d = " ".join(a.split()).split(None, 2)[2]
                if len(d):
                    sub.SetItem(index, 6, d)
                else:
                    sub.SetItem(index, 6, '')
                index += 1

        # ----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_BUTTON, self.on_close, button_close)
        self.Bind(wx.EVT_CLOSE, self.on_close)  # controlla la chiusura (x)

    # ----------------------Event handler (callback)----------------------#

    def on_close(self, event):
        """
        Destroy this window
        """
        if self.option == '-encoders':
            pub.sendMessage("DESTROY_ORPHANED_WINDOWS", msg='FFmpegCodecs')

        elif self.option == '-decoders':
            pub.sendMessage("DESTROY_ORPHANED_WINDOWS", msg='FFmpegDecoders')



# -*- coding: UTF-8 -*-
"""
FileName: miscell.py
Porpose: Contains various functionalities for A/V Conversions
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.13.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx


class Miscellaneous(wx.Panel):
    """
    This Panel implements various functionalities
    controls for A/V Conversions.
    """

    def __init__(self, parent, opt):
        """
        This is a child of `nb_misc` of AV_Conv` class-panel (parent).
        """
        get = wx.GetApp()
        self.appdata = get.appset
        self.parent = parent  # parent is the `nb_misc` here.
        self.opt = opt

        wx.Panel.__init__(self, parent, -1,
                          style=wx.TAB_TRAVERSAL
                          | wx.BORDER_NONE,
                          name="Miscellaneous panel",
                          )
        sizerbase = wx.BoxSizer(wx.VERTICAL)
        self.labinfo = wx.StaticText(self, wx.ID_ANY, label="",
                                     style=wx.ALIGN_CENTRE_HORIZONTAL)
        sizerbase.Add(self.labinfo, 0, wx.EXPAND | wx.ALL, 2)
        if self.appdata['ostype'] == 'Darwin':
            self.labinfo.SetFont(wx.Font(14, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        else:
            self.labinfo.SetFont(wx.Font(11, wx.DEFAULT, wx.NORMAL, wx.BOLD))

        sizerbase.Add((0, 15), 0)
        gridsub = wx.BoxSizer(wx.HORIZONTAL)
        sizerbase.Add(gridsub, 0, wx.ALL | wx.CENTRE, 5)

        msg = _('Subtitle Mapping')
        txtSubmap = wx.StaticText(self, wx.ID_ANY, (msg))
        gridsub.Add(txtSubmap, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        self.cmb_Submap = wx.ComboBox(self, wx.ID_ANY,
                                      choices=[('None'),
                                               ('All'),
                                               ],
                                      size=(120, -1), style=wx.CB_DROPDOWN
                                      | wx.CB_READONLY,
                                      )
        gridsub.Add(self.cmb_Submap, 0, wx.ALL, 5)

        boxmetad = wx.BoxSizer(wx.HORIZONTAL)
        sizerbase.Add(boxmetad, 0, wx.ALL | wx.CENTRE, 5)
        self.ckbx_chap = wx.CheckBox(self, wx.ID_ANY, (_('Copy Chapters')))
        boxmetad.Add(self.ckbx_chap, 0, wx.LEFT, 5)
        msg = _('Copy Metadata')
        self.ckbx_metad = wx.CheckBox(self, wx.ID_ANY, (msg))
        boxmetad.Add(self.ckbx_metad, 0, wx.LEFT, 20)

        self.SetSizerAndFit(sizerbase)

        tip = (_('Select "All" to include any source file subtitles in the '
                 'output video.\n\nSelect "None" to exclude any subtitles '
                 'stream in the output video.\n\nThis option is '
                 'automatically ignored for output audio files.'))
        self.cmb_Submap.SetToolTip(tip)
        tip = (_('Copy the chapter markers as is from source file. This '
                 'option is automatically ignored for output audio files.'))
        self.ckbx_chap.SetToolTip(tip)
        tip = (_('Copy all incoming metadata from source file, such as '
                 'audio/video tags, titles, unique marks, and so on.'))
        self.ckbx_metad.SetToolTip(tip)

        self.Bind(wx.EVT_CHECKBOX, self.on_chapters, self.ckbx_chap)
        self.Bind(wx.EVT_COMBOBOX, self.on_subtitles, self.cmb_Submap)
        self.Bind(wx.EVT_CHECKBOX, self.on_metadata, self.ckbx_metad)

        self.default()

    def default(self):
        """
        Reset all controls to default
        """

        msg = _("Additional options")
        self.labinfo.SetLabel(msg)

        self.cmb_Submap.SetSelection(1), self.on_subtitles(None)
        self.ckbx_chap.SetValue(True), self.on_chapters(None)
        self.ckbx_metad.SetValue(True), self.on_metadata(None)
    # ------------------------------------------------------------------#

    def on_subtitles(self, event):
        """
        Event on combobox list for subtitles
        """
        smap = self.cmb_Submap.GetValue()
        if smap == 'None':
            self.opt["SubtitleMap"] = '-sn'
        elif smap == 'All':
            self.opt["SubtitleMap"] = '-map 0:s?'
    # ------------------------------------------------------------------#

    def on_chapters(self, event):
        """
        Event on enabling/disabling map chapters
        """
        val = self.ckbx_chap.GetValue()
        self.opt["Chapters"] = '-map_chapters 0' if val else '-map_chapters -1'
    # ------------------------------------------------------------------#

    def on_metadata(self, event):
        """
        Event on enabling/disabling metadata
        """
        val = self.ckbx_metad.GetValue()
        self.opt["MetaData"] = '-map_metadata 0' if val else '-map_metadata -1'
    # ------------------------------------------------------------------#



# -*- coding: UTF-8 -*-
"""
Name: ffprobe.py
Porpose: simple cross-platform wrap for ffprobe
Compatibility: Python3
Platform: all platforms
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Feb.17.2022
Code checker: flake8, pylint

This file is part of FFcuesplitter.

   FFcuesplitter is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   FFcuesplitter is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FFcuesplitter.  If not, see <http://www.gnu.org/licenses/>.
"""
import subprocess
import shlex
import platform
import json
from videomass.vdms_utils.utils import Popen


def from_kwargs_to_args(kwargs):
    """
    Helper function to build command line
    arguments out of dict.
    """
    args = []
    for key in sorted(kwargs.keys()):
        val = kwargs[key]
        args.append(f'-{key}')
        if val is not None:
            args.append(f'{val}')
    return args


def ffprobe(filename, cmd='ffprobe', txtenc='utf-8', **kwargs):
    """
    Run ffprobe subprocess on the specified file.
    This function always returns a tuple of two items (data, error),
    where `data` is the data representation given from the subprocess
    output, and `error` is the current status error.

    Raises:
        `OSError` or `FileNotFoundError` occurs if the ffprobe
        binary/executable does not exists.
    Returns:
        If the above exceptions raises, or if ffprobe returns a
        non-zero exit code, Returns (None, str(error)).
        Returns a JSON representation (dict(data), None) of the
        subprocess output otherwise.
    Usage:
        >>> from videomass.vdms_threads.ffprobe import ffprobe
        >>> probe = ffprobe(filename,
                            cmd='/path/to/ffprobe',
                            loglevel='error',
                            hide_banner=None,
                            **kwargs,
                            )
        >>> if probe[1]:
                raise Exception(probe[1])
        >>> else:
        >>>     probe[0]
    """
    args = (f'"{cmd}" -show_format -show_streams -of json '
            f'{" ".join(from_kwargs_to_args(kwargs))} '
            f'"{filename}"'
            )
    args = shlex.split(args) if platform.system() != 'Windows' else args
    try:
        with Popen(args,
                   stdout=subprocess.PIPE,
                   stderr=subprocess.PIPE,
                   bufsize=1,
                   universal_newlines=True,
                   encoding=txtenc,
                   ) as proc:
            output, error = proc.communicate()

            if proc.returncode != 0:
                return (None, f'ffprobe: {error}')

    except (OSError, FileNotFoundError, UnicodeDecodeError) as excepterr:
        return (None, excepterr)

    return json.loads(output), None



# -*- coding: UTF-8 -*-
"""
Name: mediainfo.py
Porpose: show media streams information through ffprobe data
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: July.17.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx
from pubsub import pub


class MediaStreams(wx.Dialog):
    """
    Display streams information using ffprobe json data.
    """

    def __init__(self, data, OS):
        """
        list(data):
            contains ffprobe data from `MainFrame.self.data_files`.
        """
        self.data = data
        get = wx.GetApp()  # get data from bootstrap
        if get.appset['IS_DARK_THEME'] is True:
            self.mark = '#174573'
        elif get.appset['IS_DARK_THEME'] is False:
            self.mark = '#3bbbe6'
        else:
            self.mark = '#808080'
        self.colorscheme = get.appset['colorscheme']
        vidicon = get.iconset['videomass']

        wx.Dialog.__init__(self, None,
                           style=wx.DEFAULT_DIALOG_STYLE
                           | wx.RESIZE_BORDER
                           | wx.DIALOG_NO_PARENT
                           )
        sizerBase = wx.BoxSizer(wx.VERTICAL)
        self.file_select = wx.ListCtrl(self,
                                       wx.ID_ANY,
                                       style=wx.LC_REPORT
                                       | wx.SUNKEN_BORDER
                                       | wx.LC_SINGLE_SEL,
                                       )
        sizerBase.Add(self.file_select, 0, wx.ALL | wx.EXPAND, 5)

        notebook = wx.Notebook(self, wx.ID_ANY)
        grid_notebook = wx.GridSizer(1, 1, 0, 0)
        grid_notebook.Add(notebook, 1, wx.ALL | wx.EXPAND, 5)
        sizerBase.Add(grid_notebook, 1, wx.EXPAND, 0)
        #  tab format
        nb_panel_1 = wx.Panel(notebook, wx.ID_ANY)
        self.format_ctrl = wx.ListCtrl(nb_panel_1,
                                       wx.ID_ANY,
                                       style=wx.LC_REPORT
                                       | wx.SUNKEN_BORDER
                                       | wx.LC_SINGLE_SEL,
                                       name='format',
                                       )
        sizer_tab1 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab1.Add(self.format_ctrl, 1, wx.ALL | wx.EXPAND, 5)
        nb_panel_1.SetSizer(sizer_tab1)
        notebook.AddPage(nb_panel_1, (_("Format")))
        #  tab video
        nb_panel_2 = wx.Panel(notebook, wx.ID_ANY)

        self.video_ctrl = wx.ListCtrl(nb_panel_2,
                                      wx.ID_ANY,
                                      style=wx.LC_REPORT
                                      | wx.SUNKEN_BORDER
                                      | wx.LC_SINGLE_SEL,
                                      name='video'
                                      )
        sizer_tab2 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab2.Add(self.video_ctrl, 1, wx.ALL | wx.EXPAND, 5)
        nb_panel_2.SetSizer(sizer_tab2)
        notebook.AddPage(nb_panel_2, (_("Video Stream")))
        #  tab audio
        nb_panel_3 = wx.Panel(notebook, wx.ID_ANY)
        self.audio_ctrl = wx.ListCtrl(nb_panel_3,
                                      wx.ID_ANY,
                                      style=wx.LC_REPORT
                                      | wx.SUNKEN_BORDER
                                      | wx.LC_SINGLE_SEL,
                                      name='audio'
                                      )
        sizer_tab3 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab3.Add(self.audio_ctrl, 1, wx.ALL | wx.EXPAND, 5)
        nb_panel_3.SetSizer(sizer_tab3)
        notebook.AddPage(nb_panel_3, (_("Audio Streams")))
        #  tab subtitle
        nb_panel_4 = wx.Panel(notebook, wx.ID_ANY)
        self.subt_ctrl = wx.ListCtrl(nb_panel_4,
                                     wx.ID_ANY,
                                     style=wx.LC_REPORT
                                     | wx.SUNKEN_BORDER
                                     | wx.LC_SINGLE_SEL,
                                     name='subtitle'
                                     )
        sizer_tab4 = wx.BoxSizer(wx.VERTICAL)
        sizer_tab4.Add(self.subt_ctrl, 1, wx.ALL | wx.EXPAND, 5)
        nb_panel_4.SetSizer(sizer_tab4)
        notebook.AddPage(nb_panel_4, (_("Subtitle Streams")))
        # ----- confirm buttons section
        gridbtns = wx.GridSizer(1, 2, 0, 0)
        gridclip = wx.GridSizer(1, 1, 0, 0)
        self.btn_copyclip = wx.Button(self, wx.ID_ANY, _("Copy to clipboard"))
        self.btn_copyclip.Disable()
        gridclip.Add(self.btn_copyclip, 0, wx.ALL, 5)
        gridbtns.Add(gridclip)
        boxaff = wx.BoxSizer(wx.HORIZONTAL)
        btn_close = wx.Button(self, wx.ID_CLOSE, "")
        boxaff.Add(btn_close, 0)
        gridbtns.Add(boxaff, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=5)
        sizerBase.Add(gridbtns, 0, wx.EXPAND)

        # ----------------------Properties----------------------#
        self.file_select.InsertColumn(0, _('FILE SELECTION'), width=600)

        self.format_ctrl.InsertColumn(0, _('Properties'), width=200)
        self.format_ctrl.InsertColumn(1, _('Values'), width=400)

        self.video_ctrl.InsertColumn(0, _('Properties'), width=200)
        self.video_ctrl.InsertColumn(1, _('Values'), width=400)

        self.audio_ctrl.InsertColumn(0, _('Properties'), width=200)
        self.audio_ctrl.InsertColumn(1, _('Values'), width=400)

        self.subt_ctrl.InsertColumn(0, _('Properties'), width=200)
        self.subt_ctrl.InsertColumn(1, _('Values'), width=400)

        # set layout
        self.SetTitle(_('Streams Properties'))
        self.SetMinSize((700, 600))
        # self.file_select.SetMinSize((-1, 150))
        self.SetSizer(sizerBase)
        # sizerBase.Fit(self)
        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(vidicon, wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        self.Fit()
        self.Layout()
        self.CentreOnScreen()

        flist = [x['format']['filename'] for x in self.data
                 if x['format']['filename']]
        index = 0
        for files in flist:
            self.file_select.InsertItem(index, files)
            index += 1
        # ----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.file_select)

        self.format_ctrl.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_selected)
        self.video_ctrl.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_selected)
        self.audio_ctrl.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_selected)
        self.subt_ctrl.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_selected)

        self.format_ctrl.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.on_deselected)
        self.video_ctrl.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.on_deselected)
        self.audio_ctrl.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.on_deselected)
        self.subt_ctrl.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.on_deselected)

        self.Bind(wx.EVT_BUTTON, self.on_copy_to_clipboard, self.btn_copyclip)
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_close)
        self.Bind(wx.EVT_CLOSE, self.on_close)  # controlla la chiusura (x)

        if data:
            self.file_select.Focus(0)  # make the line the current line
            self.file_select.Select(0, on=1)  # default event selection

        self.typelist = None

    # ----------------------Event handler (callbacks)----------------------#

    def on_deselected(self, event):
        """
        Event on deselecting an item on the list
        """
        self.btn_copyclip.Disable()
        self.typelist = None
    # ----------------------------------------------------------------------

    def on_selected(self, event):
        """
        Event on selecting an item on the list
        """
        if not event.GetText():
            self.typelist = None
            return
        self.typelist = event.GetEventObject().Name
        self.btn_copyclip.Enable()
    # ----------------------------------------------------------------------

    def on_copy_to_clipboard(self, event):
        """
        Click on copy to clipboard button, evaluate which
        checklist you are using, then call the appropriate method.
        """
        if self.typelist == 'format':
            self.on_format()
        elif self.typelist == 'video':
            self.on_video()
        elif self.typelist == 'audio':
            self.on_audio()
        elif self.typelist == 'subtitle':
            self.on_sub()
    # ----------------------------------------------------------------------

    def on_format(self):
        """
        Copy selected item to clipboard
        """
        item = self.format_ctrl.GetFocusedItem()
        if item == -1:
            return
        col0 = self.format_ctrl.GetItemText(item, col=0)
        col1 = self.format_ctrl.GetItemText(item, col=1)
        text = f'{col0}: {col1}'
        if wx.TheClipboard.Open():
            data_object = wx.TextDataObject(text)
            wx.TheClipboard.SetData(data_object)
            wx.TheClipboard.Close()
        else:
            wx.MessageBox(_('Unable to open the clipboard on Format tab'),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
    # ----------------------------------------------------------------------

    def on_video(self):
        """
        Copy selected item to clipboard
        """
        item = self.video_ctrl.GetFocusedItem()
        if item == -1:
            return
        col0 = self.video_ctrl.GetItemText(item, col=0)
        col1 = self.video_ctrl.GetItemText(item, col=1)
        text = f'{col0}: {col1}'
        if wx.TheClipboard.Open():
            data_object = wx.TextDataObject(text)
            wx.TheClipboard.SetData(data_object)
            wx.TheClipboard.Close()
        else:
            wx.MessageBox(_('Unable to open the clipboard on '
                            'Video Streams tab'),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
    # ----------------------------------------------------------------------

    def on_audio(self):
        """
        Copy selected item to clipboard
        """
        item = self.audio_ctrl.GetFocusedItem()
        if item == -1:
            return
        col0 = self.audio_ctrl.GetItemText(item, col=0)
        col1 = self.audio_ctrl.GetItemText(item, col=1)
        text = f'{col0}: {col1}'
        if wx.TheClipboard.Open():
            data_object = wx.TextDataObject(text)
            wx.TheClipboard.SetData(data_object)
            wx.TheClipboard.Close()
        else:
            wx.MessageBox(_('Unable to open the clipboard on '
                            'Audio Streams tab'),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
    # ----------------------------------------------------------------------

    def on_sub(self):
        """
        Copy selected item to clipboard
        """
        item = self.subt_ctrl.GetFocusedItem()
        if item == -1:
            return
        col0 = self.subt_ctrl.GetItemText(item, col=0)
        col1 = self.subt_ctrl.GetItemText(item, col=1)
        text = f'{col0}: {col1}'
        if wx.TheClipboard.Open():
            data_object = wx.TextDataObject(text)
            wx.TheClipboard.SetData(data_object)
            wx.TheClipboard.Close()
        else:
            wx.MessageBox(_('Unable to open the clipboard on '
                            'Subtitle Streams tab'),
                          _('Videomass - Error!'), wx.ICON_ERROR, self)
    # ----------------------------------------------------------------------

    def on_select(self, event):
        """
        Update and populate all listctrls during items selection
        on self.file_select list control
        """
        # delete previous append:
        self.format_ctrl.DeleteAllItems()
        self.video_ctrl.DeleteAllItems()
        self.audio_ctrl.DeleteAllItems()
        self.subt_ctrl.DeleteAllItems()

        index = self.file_select.GetFocusedItem()
        item = self.file_select.GetItemText(index)

        index = 0

        for x in self.data:
            if x.get('format').get('filename') == item:
                select = self.data[self.data.index(x)]
                for k, v in x.get('format').items():
                    self.format_ctrl.InsertItem(index, str(k))
                    self.format_ctrl.SetItem(index, 1, str(v))
                    index += 1

        if select.get('streams'):
            index = 0
            for t in select.get('streams'):
                if t.get('codec_type') == 'video':
                    for k, v in t.items():
                        self.video_ctrl.InsertItem(index, str(k))
                        self.video_ctrl.SetItem(index, 1, str(v))
                        if k == 'index':
                            self.video_ctrl.SetItemBackgroundColour(index,
                                                                    self.mark)
                        index += 1
            index = 0
            for t in select.get('streams'):
                if t.get('codec_type') == 'audio':
                    for k, v in t.items():
                        self.audio_ctrl.InsertItem(index, str(k))
                        self.audio_ctrl.SetItem(index, 1, str(v))
                        if k == 'index':
                            self.audio_ctrl.SetItemBackgroundColour(index,
                                                                    self.mark)
                        index += 1
            index = 0
            for t in select.get('streams'):
                if t.get('codec_type') == 'subtitle':
                    for k, v in t.items():
                        self.subt_ctrl.InsertItem(index, str(k))
                        self.subt_ctrl.SetItem(index, 1, str(v))
                        if k == 'index':
                            self.subt_ctrl.SetItemBackgroundColour(index,
                                                                   self.mark)
                        index += 1

        self.btn_copyclip.Disable()
        self.typelist = None
    # ----------------------------------------------------------------------

    def on_close(self, event):
        """
        Destroy this dialog
        """
        pub.sendMessage("DESTROY_ORPHANED_WINDOWS", msg='MediaStreams')



# -*- coding: UTF-8 -*-
"""
Name: setting_profiles.py
Porpose: profiles storing and profiles editing dialog
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Apr.06.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import webbrowser
import wx
from videomass.vdms_utils.presets_manager_utils import write_new_profile
from videomass.vdms_utils.presets_manager_utils import edit_existing_profile


class SettingProfile(wx.Dialog):
    """
    Show dialog to store and edit profiles of a selected preset.

    """
    PASS_1 = _("One-Pass, Do not start with `ffmpeg "
               "-i filename`; do not end with "
               "`output-filename`"
               )
    PASS_2 = _("Two-Pass (optional), Do not start with "
               "`ffmpeg -i filename`; do not end with "
               "`output-filename`"
               )
    SUPFORMAT = _("Supported Formats list (optional). Do not include the `.`")
    OUTFORMAT = _("Output Format. Empty to copy format and codec. "
                  "Do not include the `.`")
    # ------------------------------------------------------------------

    def __init__(self, parent, arg, filename, array, title):
        """
        arg: evaluate if this dialog is used for add new profile or
             edit a existing profiles from three message strings:
        arg = 'newprofile'  from preset manager
        arg = 'edit' from preset manager
        arg = 'addprofile' from video and audio conversions

        """
        get = wx.GetApp()
        self.appdata = get.appset
        self.path_prst = os.path.join(self.appdata['confdir'], 'presets',
                                      f'{filename}.json')
        self.arg = arg  # evaluate if 'edit', 'newprofile', 'addprofile'
        self.array = array  # param list [name,descript,cmd1,cmd2,supp,ext,..]

        wx.Dialog.__init__(self, parent, -1, title,
                           style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)

        size_base = wx.BoxSizer(wx.VERTICAL)
        labname = wx.StaticText(self, wx.ID_ANY, _("Profile Name"))
        size_base.Add(labname, 0, wx.ALL | wx.EXPAND, 5)
        self.txt_name = wx.TextCtrl(self, wx.ID_ANY, "")
        size_base.Add(self.txt_name, 0, wx.ALL | wx.EXPAND, 5)
        labdescr = wx.StaticText(self, wx.ID_ANY, _("Description"))
        size_base.Add(labdescr, 0, wx.ALL | wx.EXPAND, 5)
        self.txt_descript = wx.TextCtrl(self, wx.ID_ANY, "", size=(-1, 80),
                                        style=wx.TE_MULTILINE)
        size_base.Add(self.txt_descript, 0, wx.ALL | wx.EXPAND, 5)
        labpass1 = wx.StaticText(self, wx.ID_ANY, SettingProfile.PASS_1)
        size_base.Add(labpass1, 0, wx.ALL | wx.EXPAND, 5)
        self.pass_1_cmd = wx.TextCtrl(self, wx.ID_ANY, "",
                                      style=wx.TE_MULTILINE
                                      )
        size_base.Add(self.pass_1_cmd, 1, wx.ALL | wx.EXPAND, 5)
        labpass2 = wx.StaticText(self, wx.ID_ANY, SettingProfile.PASS_2)
        size_base.Add(labpass2, 0, wx.ALL | wx.EXPAND, 5)
        self.pass_2_cmd = wx.TextCtrl(self, wx.ID_ANY, "",
                                      style=wx.TE_MULTILINE
                                      )
        size_base.Add(self.pass_2_cmd, 1, wx.ALL | wx.EXPAND, 5)
        boxpre1 = wx.BoxSizer(wx.VERTICAL)
        msg = _("Pre-input arguments for One-Pass (optional)")
        labpre1 = wx.StaticText(self, wx.ID_ANY, msg)
        boxpre1.Add(labpre1, 0, wx.ALL | wx.EXPAND, 5)
        self.pass_1_pre = wx.TextCtrl(self, wx.ID_ANY, "")
        boxpre1.Add(self.pass_1_pre, 1, wx.ALL | wx.EXPAND, 5)
        boxpre2 = wx.BoxSizer(wx.VERTICAL)
        msg = _("Pre-input arguments for Two-Pass (optional)")
        labpre2 = wx.StaticText(self, wx.ID_ANY, msg)
        boxpre2.Add(labpre2, 0, wx.ALL | wx.EXPAND, 5)
        self.pass_2_pre = wx.TextCtrl(self, wx.ID_ANY, "")
        boxpre2.Add(self.pass_2_pre, 1, wx.ALL | wx.EXPAND, 5)
        boxpre = wx.BoxSizer(wx.HORIZONTAL)
        boxpre.Add(boxpre1, 1, wx.ALL | wx.EXPAND, 0)
        boxpre.Add(boxpre2, 1, wx.ALL | wx.EXPAND, 0)
        size_base.Add(boxpre, 0, wx.ALL | wx.EXPAND, 0)
        boxsupp = wx.BoxSizer(wx.VERTICAL)
        labsupp = wx.StaticText(self, wx.ID_ANY, SettingProfile.SUPFORMAT)
        boxsupp.Add(labsupp, 0, wx.ALL | wx.EXPAND, 5)
        self.txt_supp = wx.TextCtrl(self, wx.ID_ANY, size=(350, -1), value="")
        boxsupp.Add(self.txt_supp, 0, wx.ALL, 5)
        boxext = wx.BoxSizer(wx.VERTICAL)
        labext = wx.StaticText(self, wx.ID_ANY, SettingProfile.OUTFORMAT)
        boxext.Add(labext, 0, wx.ALL | wx.EXPAND, 5)
        self.txt_ext = wx.TextCtrl(self, wx.ID_ANY, "")
        boxext.Add(self.txt_ext, 0, wx.ALL | wx.SHAPED, 5)
        boxbot = wx.BoxSizer(wx.HORIZONTAL)
        boxbot.Add(boxsupp, 1, wx.ALL | wx.EXPAND, 0)
        boxbot.Add(boxext, 1, wx.ALL | wx.EXPAND, 0)
        size_base.Add(boxbot, 0, wx.ALL | wx.EXPAND, 0)
        # ----- confirm buttons section
        grdBtn = wx.GridSizer(1, 2, 0, 0)
        grdhelp = wx.GridSizer(1, 1, 0, 0)
        btn_help = wx.Button(self, wx.ID_HELP, "")
        grdhelp.Add(btn_help, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        grdBtn.Add(grdhelp)
        grdexit = wx.BoxSizer(wx.HORIZONTAL)
        btn_canc = wx.Button(self, wx.ID_CANCEL, "")
        grdexit.Add(btn_canc, 0, wx.ALIGN_CENTER_VERTICAL)
        btn_save = wx.Button(self, wx.ID_OK)
        grdexit.Add(btn_save, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        grdBtn.Add(grdexit, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=5)
        size_base.Add(grdBtn, 0, wx.EXPAND)

        # ----- set_properties:
        if self.appdata['ostype'] == 'Darwin':
            fontsize = 10
        else:
            fontsize = 8
        self.pass_1_cmd.SetFont(wx.Font(fontsize, wx.FONTFAMILY_TELETYPE,
                                        wx.NORMAL, wx.NORMAL))
        self.pass_2_cmd.SetFont(wx.Font(fontsize, wx.FONTFAMILY_TELETYPE,
                                        wx.NORMAL, wx.NORMAL))
        self.pass_1_pre.SetFont(wx.Font(fontsize, wx.FONTFAMILY_TELETYPE,
                                        wx.NORMAL, wx.NORMAL))
        self.pass_2_pre.SetFont(wx.Font(fontsize, wx.FONTFAMILY_TELETYPE,
                                        wx.NORMAL, wx.NORMAL))

        self.txt_name.SetToolTip(_('A short profile name'))
        self.txt_descript.SetToolTip(_('A long description of the profile'))
        self.pass_1_cmd.SetToolTip(_('FFmpeg arguments for one-pass encoding'))
        self.pass_2_cmd.SetToolTip(_('FFmpeg arguments for two-pass encoding'))
        self.txt_supp.SetToolTip(_('One or more comma-separated format names '
                                   'that are not compatible with this '
                                   'profile.'))
        self.txt_ext.SetToolTip(_('Output format extension. Leave empty to '
                                  'copy codec and format'))

        tip = (_('Any optional arguments to add before input file on the '
                 'one-pass encoding, e.g required names of some hardware '
                 'accelerations like -hwaccel to use with CUDA.'))
        self.pass_1_pre.SetToolTip(tip)
        tip = (_('Any optional arguments to add before input file on the '
                 'two-pass encoding, e.g required names of some hardware '
                 'accelerations like -hwaccel to use with CUDA.'))
        self.pass_2_pre.SetToolTip(tip)
        # ------ Set Layout
        self.SetMinSize((850, 600))
        self.SetSizer(size_base)
        self.Fit()
        self.Layout()

        # ----------------------Binder (EVT)----------------------#
        self.Bind(wx.EVT_TEXT, self.on_Name, self.txt_name)
        self.Bind(wx.EVT_TEXT, self.on_Descript, self.txt_descript)
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_canc)
        self.Bind(wx.EVT_BUTTON, self.on_help, btn_help)
        self.Bind(wx.EVT_BUTTON, self.on_apply, btn_save)

        # -------------------Binder (EVT) End --------------------#
        if arg == 'edit':
            self.array[5] = '' if array[5] == 'copy' else array[5]
            self.change()
        elif arg == 'addprofile':
            self.pass_1_cmd.AppendText(self.array[0])  # command or param
            self.pass_2_cmd.AppendText(self.array[1])
            self.txt_ext.AppendText(self.array[2])  # extension

    def change(self):
        """
        In edit mode only, paste the array items on text boxes

        """
        self.txt_name.AppendText(self.array[0])  # name
        self.txt_descript.AppendText(self.array[1])  # descript
        self.pass_1_cmd.AppendText(self.array[2])  # command 1
        self.pass_2_cmd.AppendText(self.array[3])  # command 2
        self.txt_supp.AppendText(self.array[4])  # file supportted
        self.txt_ext.AppendText(self.array[5])  # extension
        self.pass_1_pre.AppendText(self.array[6])  # input 1
        self.pass_2_pre.AppendText(self.array[7])  # input 2
    # ---------------------Callbacks (event handler)----------------------#

    def on_Name(self, event):
        """Set default background"""
        if self.txt_name.GetBackgroundColour() == (152, 131, 19, 255):
            # html: ('#988313') == rgb: (152, 131, 19, 255) =
            self.txt_name.SetBackgroundColour(wx.NullColour)
    # ------------------------------------------------------------------#

    def on_Descript(self, event):
        """Set default background"""
        if self.txt_descript.GetBackgroundColour() == (152, 131, 19, 255):
            # html: ('#988313') == rgb: (152, 131, 19, 255) =
            self.txt_descript.SetBackgroundColour(wx.NullColour)
    # ------------------------------------------------------------------#

    def on_help(self, event):
        """
        Open default web browser via Python Web-browser controller.
        see <https://docs.python.org/3.8/library/webbrowser.html>
        """
        page = ('https://jeanslack.github.io/Videomass/User-guide/'
                'Presets_Manager_en.pdf')

        webbrowser.open(page)
    # ------------------------------------------------------------------#

    def on_close(self, event):
        """
        Close this dialog without saving anything
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def on_apply(self, event):
        """
        Apply changes
        """
        name = self.txt_name.GetValue()
        descript = self.txt_descript.GetValue()
        pass_1 = self.pass_1_cmd.GetValue()
        pass_2 = self.pass_2_cmd.GetValue()
        file_support = self.txt_supp.GetValue().strip()
        extens = self.txt_ext.GetValue().strip()
        extens = 'copy' if not extens else extens
        preinput1 = self.pass_1_pre.GetValue()
        preinput2 = self.pass_2_pre.GetValue()

        # ---------------------------------------------------------------
        if [txt for txt in [name, descript] if txt.strip() == '']:
            if not name.strip():
                self.txt_name.SetBackgroundColour('#988313')
            if not descript.strip():
                self.txt_descript.SetBackgroundColour('#988313')

            wx.MessageBox(_("Incomplete profile assignments"),
                          _('Videomass - Warning!'), wx.ICON_WARNING, self)
            return

        if len(file_support.split()) > 1:
            supp = ''.join(file_support.split())
            if [i for i in supp.split() if ',' not in i]:
                wx.MessageBox(_("Formats must be comma-separated"),
                              _('Videomass - Warning!'), wx.ICON_WARNING, self)
                return

        if self.arg in ('newprofile', 'addprofile'):
            writenewprf = write_new_profile(self.path_prst,
                                            Name=name.strip(),
                                            Description=descript,
                                            First_pass=pass_1,
                                            Second_pass=pass_2,
                                            Supported_list=file_support,
                                            Output_extension=extens,
                                            Preinput_1=preinput1,
                                            Preinput_2=preinput2,
                                            )
            if writenewprf == 'already exist':
                wx.MessageBox(_("Profile already stored with same name"),
                              _('Videomass - Warning!'), wx.ICON_WARNING, self)
                return

            wx.MessageBox(_("Successful storing!"))

        elif self.arg == 'edit':
            editprf = edit_existing_profile(self.path_prst,
                                            self.array[0],
                                            Name=name.strip(),
                                            Description=descript,
                                            First_pass=pass_1,
                                            Second_pass=pass_2,
                                            Supported_list=file_support,
                                            Output_extension=extens,
                                            Preinput_1=preinput1,
                                            Preinput_2=preinput2,
                                            )
            if editprf == 'already exist':
                wx.MessageBox(_("Profile already stored with same name"),
                              _('Videomass - Warning!'), wx.ICON_WARNING, self)
                return
            wx.MessageBox(_("Successful changes!"))
            # self.Destroy() # con ID_OK e ID_CANCEL non serve

        event.Skip()



# -*- coding: UTF-8 -*-
"""
Name: subtitle_editor.py
Porpose: shows a dialog box to setting preferred subtitles
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: July.17.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx
from videomass.vdms_dialogs.widget_utils import NormalTransientPopup


class SubtitleEditor(wx.Dialog):
    """
    Shows a dialog box for setting preferred subtitles.
    See ``youtubedl_ui.py`` -> ``on_subtitles_editor`` method
    for how to use this class.

    """
    LGREEN = '#52ee7d'
    BLACK = '#1f1f1f'
    get = wx.GetApp()  # get data from bootstrap
    APPICON = get.iconset['videomass']
    SUBS_LANG = {"en": _("English"),
                 "fr": _("French"),
                 "de": _("German"),
                 "el": _("Greek"),
                 "he": _("Hebrew"),
                 "it": _("Italian"),
                 "nl": _("Dutch"),
                 "pl": _("Polish"),
                 "pt": _("Portuguese"),
                 "ru": _("Russian"),
                 "es": _("Spanish"),
                 "sv": _("Swedish"),
                 "tr": _("Turkish"),
                 "sq": _("Albanian"),
                 "zh": _("Chinese"),
                 }

    def __init__(self, parent, data):
        """
        NOTE Use 'parent, -1' param. to make parent, use 'None' otherwise

        """
        self.data = data

        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)

        # ------ Add widget controls
        sizbase = wx.BoxSizer(wx.VERTICAL)
        labtstr1 = _('Subtitles')
        lab = wx.StaticText(self, label=labtstr1)
        sizbase.Add(lab, 0, wx.LEFT, 5)
        lab.SetLabelMarkup(f"<b>{labtstr1}</b>")
        choicelang = (_("None"),
                      _("All available subtitles"),
                      _("Select by default language"),
                      _('Custom language subtitles'),
                      )
        self.cmbx_langs = wx.ComboBox(self, wx.ID_ANY,
                                      choices=choicelang,
                                      size=(-1, -1),
                                      style=wx.CB_DROPDOWN
                                      | wx.CB_READONLY,
                                      )
        sizbase.Add(self.cmbx_langs, 0, wx.ALL | wx.EXPAND, 5)
        choice = list(SubtitleEditor.SUBS_LANG.values())
        self.listblang = wx.ListBox(self, wx.ID_ANY,
                                    choices=choice,
                                    style=0,
                                    name='ListBox',
                                    )
        sizbase.Add(self.listblang, 1, wx.ALL | wx.EXPAND, 5)
        sizbase.Add(10, 10)
        labtstr2 = _('Custom language')
        self.lab1 = wx.StaticText(self, label=labtstr2)
        sizbase.Add(self.lab1, 0, wx.LEFT, 5)
        self.lab1.SetLabelMarkup(f"<b>{labtstr2}</b>")
        sizcustsub = wx.BoxSizer(wx.HORIZONTAL)
        sizbase.Add(sizcustsub, 0, wx.ALL | wx.EXPAND, 5)
        self.btn_help = wx.Button(self, wx.ID_ANY, _("Read me"), size=(-1, -1))
        sizcustsub.Add(self.btn_help, 0)
        self.addlangs = wx.TextCtrl(self,
                                    wx.ID_ANY,
                                    value=(""),
                                    size=(-1, -1),
                                    name="custom_subs",
                                    )
        sizcustsub.Add(self.addlangs, 1, wx.LEFT | wx.EXPAND, 5)
        sizbase.Add(10, 10)
        labtstr3 = _('Subtitle options')
        self.lab3 = wx.StaticText(self, label=labtstr3)
        sizbase.Add(self.lab3, 0, wx.LEFT, 5)
        self.lab3.SetLabelMarkup(f"<b>{labtstr3}</b>")
        txt = _('Include automatically generated subtitle (YouTube only)')
        self.ckbx_autogen = wx.CheckBox(self, wx.ID_ANY, txt)
        sizbase.Add(self.ckbx_autogen, 0, wx.ALL, 5)
        txt = _('Embed subtitles into video file')
        self.ckbx_embed = wx.CheckBox(self, wx.ID_ANY, txt)
        sizbase.Add(self.ckbx_embed, 0, wx.ALL, 5)
        txt = _('Download subtitles only (do not include audio and video)')
        self.ckbx_skip_dl = wx.CheckBox(self, wx.ID_ANY, txt)
        sizbase.Add(self.ckbx_skip_dl, 0, wx.ALL, 5)

        # ------ bottom layout buttons
        sizbott = wx.BoxSizer(wx.HORIZONTAL)
        btn_cancel = wx.Button(self, wx.ID_CANCEL, "")
        sizbott.Add(btn_cancel, 0)
        btn_ok = wx.Button(self, wx.ID_OK)
        sizbott.Add(btn_ok, 0, wx.LEFT, 5)
        sizbase.Add(sizbott, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=5)

        # ------ Properties
        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(SubtitleEditor.APPICON,
                                      wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        self.SetTitle(_('Subtitles Editor'))
        self.SetMinSize((600, 400))
        self.SetSizer(sizbase)
        sizbase.Fit(self)
        self.Layout()

        # ----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_COMBOBOX, self.on_langs, self.cmbx_langs)
        self.Bind(wx.EVT_BUTTON, self.on_help, self.btn_help)
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_cancel)
        self.Bind(wx.EVT_BUTTON, self.on_ok, btn_ok)

        if not self.data["writesubtitles"]:
            self.default_setting()
        else:
            self.ckbx_autogen.SetValue(self.data["writeautomaticsub"])
            self.ckbx_embed.SetValue(self.data["embedsubtitle"])
            self.ckbx_skip_dl.SetValue(self.data["skip_download"])
            deflang = ''.join(self.data["subtitleslangs"])

            if deflang == 'all':
                self.cmbx_langs.SetSelection(1)

            elif deflang in SubtitleEditor.SUBS_LANG:
                self.cmbx_langs.SetSelection(2)
                idx = list(SubtitleEditor.SUBS_LANG.keys()).index(deflang)
                self.listblang.SetSelection(idx)

            else:
                self.cmbx_langs.SetSelection(3)
                self.addlangs.SetValue(', '.join(self.data["subtitleslangs"]))

            self.on_langs(None)
    # ------------------------------------------------------------------#

    def default_setting(self, setcmbox=True):
        """
        Reset all settings to default (disables all controls)
        This method is called when subtitles are not required.
        """
        if setcmbox:
            self.cmbx_langs.SetSelection(0)
        self.listblang.SetSelection(0)
        self.listblang.Disable()
        self.lab1.Disable()
        self.addlangs.Clear()
        self.btn_help.Disable()
        self.btn_help.SetBackgroundColour(wx.NullColour)
        self.btn_help.SetForegroundColour(wx.NullColour)
        self.listblang.Disable()
        self.lab3.Disable()
        self.ckbx_autogen.SetValue(False)
        self.ckbx_embed.SetValue(False)
        self.ckbx_skip_dl.SetValue(False)
        self.ckbx_autogen.Disable()
        self.ckbx_embed.Disable()
        self.ckbx_skip_dl.Disable()
    # ------------------------------------------------------------------#

    def getvalue(self):
        """
        This method return values via the interface getvalue()
        by the caller. See the caller for more info and usage.
        """
        if self.cmbx_langs.GetSelection() == 0:
            writesubtitles = False
            subtitleslangs = []
            writeautomaticsub = False
            embedsubtitle = False
            skip_download = False

        elif self.cmbx_langs.GetSelection() == 1:
            writesubtitles = True
            subtitleslangs = ['all']
            writeautomaticsub = self.ckbx_autogen.GetValue()
            embedsubtitle = self.ckbx_embed.GetValue()
            skip_download = self.ckbx_skip_dl.GetValue()

        elif self.cmbx_langs.GetSelection() == 2:
            writesubtitles = True
            idx = self.listblang.GetSelection()
            lang = self.listblang.GetString(idx)
            langcode = [key for key, val in SubtitleEditor.SUBS_LANG.items()
                        if val == lang]
            subtitleslangs = langcode
            writeautomaticsub = self.ckbx_autogen.GetValue()
            embedsubtitle = self.ckbx_embed.GetValue()
            skip_download = self.ckbx_skip_dl.GetValue()

        elif self.cmbx_langs.GetSelection() == 3:
            writesubtitles = True
            txt = self.addlangs.GetValue()
            subtitleslangs = ''.join(txt.split()).split(',')
            writeautomaticsub = self.ckbx_autogen.GetValue()
            embedsubtitle = self.ckbx_embed.GetValue()
            skip_download = self.ckbx_skip_dl.GetValue()

        return {'writesubtitles': writesubtitles,
                'subtitleslangs': subtitleslangs,
                'writeautomaticsub': writeautomaticsub,
                'embedsubtitle': embedsubtitle,
                'skip_download': skip_download
                }
    # ----------------------Event handler (callback)----------------------#

    def on_help(self, event):
        """
        event on button help
        """
        msg = (_('Write in the text field on the side, one or more '
                 'language codes of the\nsubtitles to download separated by '
                 'commas, e.g. "en, ja, ar" for English,\nJapanese and '
                 'Arabic respectively.\n\n'
                 'You can use regex (regular expression), e.g. '
                 '"en.*, ja, ar.*" where the\nasterisk preceded by the dot '
                 'will allow you to download everything\n'
                 'related to the subtitles of a certain language.\n\n'
                 'You can prefix the language code with a "-" to exclude it '
                 'from the\nrequested languages, e.g. "all, -it.*, -nl.*", '
                 '-fr.*" will download all\nsubtitles except those in '
                 'Italian, Dutch and French.'))
        win = NormalTransientPopup(self,
                                   wx.SIMPLE_BORDER,
                                   msg,
                                   SubtitleEditor.LGREEN,
                                   SubtitleEditor.BLACK)

        # Show the popup right below or above the button
        # depending on available screen space...
        btn = event.GetEventObject()
        pos = btn.ClientToScreen((0, 0))
        sz = btn.GetSize()
        win.Position(pos, (0, sz[1]))

        win.Popup()
    # --------------------------------------------------------------#

    def on_langs(self, event):
        """
        ComboBox event
        """
        if self.cmbx_langs.GetSelection() == 0:
            self.default_setting(setcmbox=False)
            return

        self.lab3.Enable()
        self.ckbx_autogen.Enable()
        self.ckbx_embed.Enable()
        self.ckbx_skip_dl.Enable()

        if self.cmbx_langs.GetSelection() == 1:
            self.listblang.Disable()
            self.lab1.Disable()
            self.addlangs.Disable()
            self.btn_help.Disable()
            self.btn_help.SetBackgroundColour(wx.NullColour)
            self.btn_help.SetForegroundColour(wx.NullColour)

        elif self.cmbx_langs.GetSelection() == 2:
            self.listblang.Enable()
            self.lab1.Disable()
            self.addlangs.Disable()
            self.btn_help.Disable()
            self.btn_help.SetBackgroundColour(wx.NullColour)
            self.btn_help.SetForegroundColour(wx.NullColour)

        elif self.cmbx_langs.GetSelection() == 3:
            self.listblang.Disable()
            self.lab1.Enable()
            self.addlangs.Enable()
            self.btn_help.Enable()
            self.btn_help.SetBackgroundColour(wx.Colour(SubtitleEditor.LGREEN))
            self.btn_help.SetForegroundColour(wx.Colour(SubtitleEditor.BLACK))
    # ------------------------------------------------------------------#

    def on_close(self, event):
        """
        Close this dialog without saving anything
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def on_ok(self, event):
        """
        Don't use self.Destroy() in this dialog
        """
        event.Skip()
